/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef G_SAMPLE_CHANNEL_H
#define G_SAMPLE_CHANNEL_H


#include <functional>
#include <samplerate.h>
#include "channel.h"


class Patch;
class Wave;


class SampleChannel : public Channel
{
private:

	

	int fillChan(float* dest, int start, int offset, bool rewind=true);

	

	void clearChan(float* dest, int start);

	

	void calcFadeoutStep();

	

	void calcVolumeEnv(int frame);

	

	SRC_STATE* rsmp_state;
	SRC_DATA   rsmp_data;

	

	float* pChan;

	

	float* vChanPreview;

	

	int frameRewind;

	int   begin;
	int   end;
	float boost;
	float pitch;
	int   trackerPreview;  
	int   shift;

	

	std::function<void()> onPreviewEnd;

public:

	SampleChannel(int bufferSize, bool inputMonitor);
	~SampleChannel();

	void copy(const Channel* src, pthread_mutex_t* pluginMutex) override;
	void clear() override;
	void process(float* outBuffer, float* inBuffer) override;
	void preview(float* outBuffer) override;
	void start(int frame, bool doQuantize, int quantize, bool mixerIsRunning,
		bool forceStart, bool isUserGenerated) override;
	void kill(int frame) override;
	void empty() override;
	void stopBySeq(bool chansStopOnSeqHalt) override;
	void stop() override;
	void rewind() override;
	void setMute(bool internal) override;
	void unsetMute(bool internal) override;
  int readPatch(const std::string& basePath, int i, pthread_mutex_t* pluginMutex,
    int samplerate, int rsmpQuality) override;
	int writePatch(int i, bool isProject) override;
	void quantize(int index, int localFrame) override;
	void onZero(int frame, bool recsStopOnChanHalt) override;
	void onBar(int frame) override;
	void parseAction(giada::m::recorder::action* a, int localFrame, int globalFrame,
			int quantize, bool mixerIsRunning) override;
	bool canInputRec() override;
	bool allocBuffers() override;

	int getTrackerPreview() const;
	int getShift() const;
	void setShift(int s);

	void reset(int frame);

	

	void setFadeIn(bool internal);
	void setFadeOut(int actionPostFadeout);
	void setXFade(int frame);

	

	void pushWave(Wave* w);

	

	int getPosition();

	

	void sum(int frame, bool running);


	void setPitch(float v);
	float getPitch();
	void setBegin(int f);
	int getBegin();
	void setEnd(int f);
	int getEnd();
	void setTrackerPreview(int f);

	

	void hardStop(int frame);

	

	void setReadActions(bool v, bool recsStopOnChanHalt);

	void setBoost(float v);
	float getBoost();	

	void setOnEndPreviewCb(std::function<void()> f);

	Wave* wave;
	int   tracker;         
	int   mode;            
	bool  qWait;           
	bool  fadeinOn;
	float fadeinVol;
	bool  fadeoutOn;
	float fadeoutVol;      
	int   fadeoutTracker;  
	float fadeoutStep;     
  int   fadeoutType;     
  int		fadeoutEnd;      
  bool  inputMonitor;

	

  uint32_t midiInReadActions;
  uint32_t midiInPitch;

	

	enum {
		DO_STOP   = 0x01,
		DO_MUTE   = 0x02,
		DO_MUTE_I = 0x04
	};

	

	enum {
		FADEOUT = 0x01,
		XFADE   = 0x02
	};
};

#endif
